from imports import * 
from classes import * 
from support import * 
import getpass
from cryptography.fernet import Fernet

warnings.filterwarnings('ignore', category=UserWarning)
CONFIG_SETTINGS = dict(auto_envvar_prefix='config')

pass_config = click.make_pass_decorator(config, ensure=True)

@click.group()
@pass_config
def kli(config):
    """
    A unofficial Kaggle CLI with a few more features
    """
    
@kli.command()
@pass_config
def setup(config):
    """
    Helps setup login details 
    """
    opt = {}
    
    username = input('UserName: ')
    password = getpass.getpass(prompt='Password: ')
    password = str.encode(password)
    
    # need to make sure that config file does not already 
    # exist, and if it does, check its validity.
    #config_file = config.config_file
    #key = str.encode(config_file['key'])
    
    cipher_suite = Fernet(key)
    cipher_text = cipher_suite.encrypt(password)
    cipher_text = cipher_text.decode()
    
    opt['key'] = 'hpm3j6mDfpL9yMEOkLwMSC2Qwa2jKyEnGeI08yNcv1I='
    opt['UserName'] = username
    opt['Password'] = cipher_text
    opt['action'] = 'login'
    
    with open(config_path, 'w') as f:
        json.dump(opt, f)
    
    print('Done')
    sys.exit()
    
@kli.command()
@click.option('--comp', default=None, type=str, help='Needs a competiton name to download either training or test Set')
@click.option('--tr', default=False, is_flag=True, help='This option downloads training set only')
@click.option('--te', default=False, is_flag=True, help='This option downloads test set only')
@pass_config
def dl(config, comp, tr=False, te=False):
    """
    Downloads competition test or train data, 
    default downloads both (not recommended if time is of essence)
    """
    comp_exists = cexist(config, comp)
    all_competitions = config.comp_file

    if comp_exists:
        print("""Competition Exists...""")
        with requests.Session() as session:
            logged_in = login(config, session, root_url, test_url)
            username = config.config_file['UserName']
            if logged_in:
                print("""Logged in as %s...""" % username)
                comp_url = str(all_competitions[comp][0])
                rules_accepted = crules(session, root_url, comp_url, rules)
                if rules_accepted:
                    print("""Rules accepted...""")
                    data_response = session.get(root_url + comp_url + '/data') 
                    soup = BeautifulSoup(data_response.text, 'lxml')
                    testSetUrl, trainingSetUrl = get_links(soup, all_competitions[comp][0][3::])
                    if te:
                        print("""Downloading Test set...""")
                        download_url(session, testSetUrl)
                        sys.exit()
                    if tr:
                        print("""Downloading Training set...""")
                        download_url(session, trainingSetUrl)
                        sys.exit()
                    if tr==False and te==False:
                        print("""No training or test option selected. Downloading both...""")
                        
                        print("""Downloading Training set...""")
                        download_url(session, trainingSetUrl)
                        
                        print("""Downloading Training set...""")
                        download_url(session, testSetUrl)
                else:
                    print("""
                    An Error occured whilst retrieving the page. 
                    Perhaps the rules for this competion have not been accepted. 
                    The rules can be accepted here %s
                    """ % (root_url + comp_url + rules) )
            else:
                print("""
                    Not logged in, perhaps due to an error in your username/password.
                    You can overwrite your current user settings via the command 'kl setup'.
                        """)
                return(0)
    else:
        print("""
                    No Competition mentioned, the command can be used as 'kl dl --comp <competion name>...'.
                    A list of all competitions can be found by executing 'kl list'
        """)
        return(0)

#@kli.command()
#@pass_config
#def list(config):
#    """
#    Lists all competition names
#    """
#    comp_data = config.comp_load()
#    with open('data/keys.txt', 'w') as f:
#        for key in comp_data.keys():
#            w = key + '\n'
#            f.write(w)
#    subprocess.call(['cat keys.txt | less'], shell=True)


# all below functions are placed in order of execution in main loop (my mind works like that)
# 1. Check if the competition exists
@pass_config
def cexist(config, comp=None): 
    """
    Checks if the given competition exists within the given data set
    """
    if(comp==None):
        print("No competition name given, please try again with the --comp option")
        sys.exit()
    comp_data = config.comp_load()
    if comp in comp_data.keys():
        return(True)
    else:
        return(False)
    
# 3. if logged in, check if competition rules have been accepted by the user
# @pass_config
def crules(session, root_url, comp_url, rules):
    """
    Checks if competition rules have been accepted
    """
    response = session.get(root_url + comp_url + rules)
    soup = BeautifulSoup(response.text, 'lxml')
    hasAcceptedRules = re.findall('"hasAcceptedRules":(\w+)', str(soup))
    
    if hasAcceptedRules[0]=='true':
        return(True)
    else:
        return(False) 



from imports import * 
from classes import * 

# 2. if it exists, login
def login(config, session, root_url, test_url):
    """
    Check if the function has logged in, 
    calls another function, which checks username once logged 
    in, to ensure login and login of the right account
    """
    # may have to later change this to email or something 
    # more innocuous
    # need to login first 
    username = ['']
    login_data = config.config_file
    session.post(root_url + login_url, data=login_data)

    response = session.get(root_url + test_url)
    soup = BeautifulSoup(response.text, 'lxml')
    username = re.findall('"user_name": "(\w+)"', str(soup))
    
    if username[0]=='' or username[0]==None: 
        return(False)
    if username[0]==login_data['UserName']:
        return(True) 
    
# 4. if rules are accepted, get train and test links from soup
def get_links(soup, comp_url):
    """
    Gets Test and Train links from the site's soup
    """
    testSetUrl = ''
    trainingSetUrl = ''
    links = re.findall('"url":"(/c/{}/download/[^"]+)"'.format(comp_url), str(soup))
    
    train_exp = re.compile(r'train')
    test_exp = re.compile(r'test')

    for link in links:
        if train_exp.search(link):
            trainingSetUrl = root_url + link
        if test_exp.search(link):
            testSetUrl = root_url + link
    return(testSetUrl, trainingSetUrl)

# 5. if rules are accepted, according to the flag passed, the file is downloaded
def download_url(session, url):
    """
    Downloads test or train file: 
    extracts name from url,
    uses tqdm progressbar
    """ 
    response = session.get(url, stream=True)
    content_type = response.headers.get('Content-Type')
    
    total_length = int(response.headers.get('Content-Length'))
    
    filename = re.compile('[^/]+$').search(url).group(0)
    
    with open(filename, 'wb') as f:
        for chunk in tqdm(response.iter_content(chunk_size=1024), 
                        total=total_length/1024, 
                        unit='KB'):
            f.write(chunk)
    
    print('Done')




# for everything that the command line interface does
import click 

# for accessing websites
import requests 

# for making sure the 'list' command lists all in one page
import subprocess, argparse, argcomplete
from argcomplete.completers import ChoicesCompleter

# for parsing the soups that are thrown out of websites
from bs4 import BeautifulSoup

# for the progressbars 
from tqdm import * 

# for re for matching, sys for system access and time for timing
import re, json, warnings, sys



from setuptools import setup, find_packages

setup(
    name = 'kli', 
    version = '0.0.7', 
    include_package_data = True,
    py_modules = ['main', 'classes', 'support', 'imports'],
    packages=find_packages(),
    package_data={'':['data/*']},
    license = 'MIT',
    url = 'https://github.com/tranquilo12/kl',
    author = 'Shriram Sunder',
    author_email = 'shriram.sunder121091@gmail.com',
    install_requires = [
        'tqdm', 
        'pprint',
        'argcomplete',
        'bs4', 
        'click>=6', 
        'cryptography', 
        'requests'
        ],    
    entry_points = '''
        [console_scripts]
        kli=main:kli
    '''
#    data_files=[('data', ['comp.json']),
#                ('data', ['keys.txt'])]
        )



from imports import * 
from cryptography.fernet import Fernet

root_url = 'https://www.kaggle.com'
login_url = '/account/login'
test_url = '/c/titanic'
rules = '/rules'
data = '/data'

config_path = click.get_app_dir('kl') + '/config.json'
#comp_path = click.get_app_dir('kl') + '/comp.json'

#config_path = 'config.json'
comp_path = 'comp.json'

class config(object):
    def __init__(self, *args, **kwargs): 
        self.config_filepath = config_path
        self.config_file = self.config_load()

        self.comp_filepath = comp_path
        self.comp_file = self.comp_load()
        super(config, self).__init__()

    def config_load(self):
        with open(self.config_filepath, 'r') as f:
            try:
                d = json.load(f)
            except (ValueError, KeyError) as e:
                    opt = {}
                    key = 'hpm3j6mDfpL9yMEOkLwMSC2Qwa2jKyEnGeI08yNcv1I='
                    opt['key'] = key
                    opt['Password'] = 'gAAAAABZC018YC8y4q9gdnI-xIuNXeySuP-hOIqkqTkKxAiSQOZvGPToU8wan6_xgm8bXk-KdrBdJjsdhGOnDv1PC5ipU5FbqQ=='
                    opt['UserName'] = 'UserName'
                    opt['action'] = 'login'
                    
                    with open(self.config_filepath, 'w') as fp:
                        json.dump(opt, fp)
                        
                    print("""
                    The config_file was restored to default (or a new config_file was just made). 
                    You can reset your settings via the command 'kl setup'.
                    """)
                    
                    with open(self.config_filepath, 'r') as fp: 
                        d = json.load(fp)
                    
            try: 
                cipher_suite = Fernet(str.encode(d['key']))
            except KeyError: 
                d['key'] = 'hpm3j6mDfpL9yMEOkLwMSC2Qwa2jKyEnGeI08yNcv1I='
                cipher_suite = Fernet(str.encode(d['key']))
                
            plain_text = cipher_suite.decrypt(str.encode(d['Password']))
            d['Password'] = plain_text
            return(d)

    def comp_load(self):
        with open(self.comp_filepath) as f:
            d = json.load(f)
            return(d)
